clear;
clc;
// Se carga la librería de funciones para el diagrama de Bode sintótico.
// L librería debe estar en el directorio de operació. 
pth=pwd();
pth=pth+"\T_d_Control_lib.sci";
exec(pth);

// PÉNDULO INVERTIDO SOBRE UN CARRO

M = 0.5;
m = 0.2;
B = 0.1;
J = 0.006;
g = 9.8;
L = 0.3;
Ts=0.02;
s=%s;
p = J*(M+m)+M*m*L^2; // denominador para las matrices A y B

Amat = [0,1,0,0;0,-(J+m*L^2)*B/p,-(m^2*g*L^2)/p,0;0,0,0,1;0,(m*L*B)/p,m*g*L*(M+m)/p,0];
Bmat = [0;(J+m*L^2)/p;0;-m*L/p];
Cmat = [1 0 0 0; 0 0 1 0];
Dmat = [0;0];
// Modelo continuo lineal
P1 =syslin("c",Amat,Bmat,Cmat,Dmat)
// Autovalores 
autov=spec(Amat)
// Transferencia
zpk(P1)
// Respuesta a Lazo Abierto
t=0:0.01:1;
Y=csim('impulse',t,P1);
// GRÁFICO VARIABLES DE ESTADO A LAZO CERRADO
xdel(winsid());
subplot(2,1,1)
plot2d(t,Y(1,:),color("red"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Posición");
subplot(2,1,2)
plot2d(t,Y(2,:),color("scilabgreen4"));
xgrid(color("gray"))
ylabel("Ángulo");
title('Respuesta al Impulso a Lazo Abierto')

// Modelo de estado discreto
Ts = 0.02;
disp('Modelo de estado discreto');
P1_dis =dscr(P1,Ts)

Ad=P1_dis.a;
Bd=P1_dis.b;
Cd=P1_dis.c;
Dd=P1_dis.d;
// Autovalores
spec(Ad)
// Controlabilidad
U=cont_mat(Ad,Bd)
rank(U)

//  PARÁMETROS PARA EL CÁLCULO DE LA REALIMENTACIÓN

/* SE DESEA PARA LAS VARIABLES DEL SISTEMA UNA RESPUESTA AL ESCALÓN CON UN SOBREPICO DEL 20% Y UN TIEMPO DE ESTABLECIMIENTO DE 5 SEGUNDOS.
SE VA A UTILIZAR UNA APROXIMACIÓN DE SEGUNDO ORDEN PARA CALCULAR LOS AUTOVALORES.
PARA EL TIEMPO DE ESTABLECIMIENTO Test=4/(Psi Wn)
PARA EL SOBREPICO SP= EXP(-Psi Pi/(1-Psi^2)^0.5) */
SP=0.1;
Test=3;
Psi= sqrt((log(SP))^2/((log(SP))^2+%pi^2))
Psi=1
Wn = 4/(Psi*Test)
p1 = -Psi*Wn+%i*Wn*sqrt(1-Psi^2)
p2 = -Psi*Wn-%i*Wn*sqrt(1-Psi^2)

/* AGREGO LOS DOS AUTOVALORES RESTANTES CON PARTE REAL DE AL MENOS 5 VECES MÁS GRANDE EN MÓDULO */

p3 = 5*(-Psi*Wn)
p4 = 10*(-Psi*Wn)

// p1 = -8.4910 + 7.9283i;
// p2 = -8.4910 - 7.9283i;
// p3 = -4.7592 + 0.8309i;
// p4 = -4.7592 - 0.8309i;

// TRANSFORMO ESTOS AUTOVALORES AL PLANO Z

Polos= [exp(p1*Ts) exp(p2*Ts) exp(p3*Ts) exp(p4*Ts)]

// DISEÑO DE LA REALIMENTACIÓN DE ESTADOS

Kt=ppol(Ad,Bd,Polos)

Alc = [(Ad-Bd*Kt)]
Blc = Bd
Clc= [eye(4,4);-Kt]
Dlc= [zeros(4,1);1]
X0=[0;0;0.1;0]
P2_dis=syslin("d",Alc,Blc,Clc,Dlc,X0); // Modelo de Lazo Cerrado
autov=spec(Alc)

//  RESPUESTA DEL SISTEMA A LAZO CERRADO
t=0:Ts:10;
//r =0.2*ones(size(t));
r=zeros(1,length(t));
Y=dsimul(P2_dis,r); // Simulación
// GRÁFICO VARIABLES DE ESTADO A LAZO CERRADO
xdel(winsid());
subplot(4,1,1)
plot2d(t,Y(1,:),color("red"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Posición lineal");
subplot(4,1,2)
plot2d(t,Y(2,:),color("scilabgreen4"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Velocidad lineal");
subplot(4,1,3)
plot2d(t,Y(3,:),color("blue"));
xgrid(color("gray"))
ylabel("Ángulo");
subplot(4,1,4)
plot2d(t,Y(4,:),color("scilabmagenta4"));
xgrid(color("gray"))
ylabel("Velocidad angular");
figure(2)
f = gcf();
f.background = 8;
plot2d(t,Y(5,:),color("scilabgreen4"));
xgrid(color("gray"))
title('Respuesta del Control')
ylabel("Control");

// Estabilidad
// se calcula la transferencia de lazo abierto GH(z)=Kt*(zI-Ad)^-1*Bd
// armo un modelo de estado con Kt como salida.
z=%z;
GH=syslin("d",Ad,Bd,Kt,[0]);
GHz=inv(z*eye(4,4)-Ad)*Bd
GHz=Kt*GHz    
zpk(GHz)
// Transformación bilineal
GHw=ss2tf(bilin(GHz,[Ts/2,1,-Ts/2,1]))
zpk(GHw)                  
roots(GHw.den)
roots(GHw.num)
// Diagramas de Bode
xdel(winsid());
wr=logspace(-3,8,500);
bode_doble(GHw,wr,0)
G0=syslin("c",(s+1.360D-10),s);
GHw=GHw*G0;

// Nyquist Logarítmico
xdel(winsid())
LogNyquist(GHw,6,2);

// Lugar de Raices
xdel(winsid())
evans(GHw)
